/*---------------------------------------------------------------------\
|                                                                      |
|                     _     _   _   _     __     _                     |
|                    | |   | | | \_/ |   /  \   | |                    |
|                    | |   | | | |_| |  / /\ \  | |                    |
|                    | |__ | | | | | | / ____ \ | |__                  |
|                    |____||_| |_| |_|/ /    \ \|____|                 |
|                                                                      |
|                             runlevel library                         |
|                                                                      |
|                                         (C) SUSE Linux Products GmbH |
\----------------------------------------------------------------------/

  File:       LSBService.hpp

  Author:     Marius Tomaschewski
  Maintainer: Marius Tomaschewski

  Purpose:

/-*/
/**
 * @file   LSBService.hpp
 * @brief  LSB Service class and other LSB related definitions.
 *
 * The Linux Standard Base (LSB) specifiation can be found at
 * http://www.linuxbase.org/spec/. This file is based on LSB
 * specification version 2.1.0.
 * Read the section 8.2, "Init-Script Actions" for more details.
 *
 * @todo update to LSB 3.0
 */
#ifndef   LIMAL_RUNLEVEL_LSB_SERVICE_HPP
#define   LIMAL_RUNLEVEL_LSB_SERVICE_HPP

#include <limal/runlevel/config.h>
#include <limal/runlevel/Service.hpp>
#include <limal/runlevel/Runlevel.hpp>
#include <blocxx/String.hpp>
#include <blocxx/Array.hpp>
#include <blocxx/Map.hpp>


// -------------------------------------------------------------------
namespace LIMAL_NAMESPACE
{
namespace RUNLEVEL_NAMESPACE
{

// -------------------------------------------------------------------
/**
 * @brief LSB service action exit code type.
 *
 * LSB defines following service action exit codes:
 * - In the case of LSB init script action other than "status",
 *   the script shall return one of the following exit codes:
 *   - @c 0            success
 *   - @c 1            generic or unspecified error
 *   - @c 2            invalid or excess argument(s)
 *   - @c 3            unimplemented feature (for example, "reload")
 *   - @c 4            user had insufficient privilege
 *   - @c 5            program is not installed
 *   - @c 6            program is not configured
 *   - @c 7            program is not running
 *   - @c 8   - @c  99 reserved for future LSB use
 *   - @c 100 - @c 149 reserved for distribution use
 *   - @c 150 - @c 199 reserved for application use
 *   - @c 200 - @c 254 reserved
 *   .
 * - In the case of LSB init script "status" action, the script
 *   shall return one of the following exit codes:
 *   - @c 0            program is running or service is OK
 *   - @c 1            program is dead and /var/run pid file exists
 *   - @c 2            program is dead and /var/lock lock file exists
 *   - @c 3            program is not running
 *   - @c 4            program or service status is unknown
 *   - @c 5   - @c  99 reserved for future LSB use
 *   - @c 100 - @c 149 reserved for distribution use
 *   - @c 150 - @c 199 reserved for application use
 *   - @c 200 - @c 254 reserved
 *   .
 * .
 */
typedef blocxx::Int32       LSBExitCode;

/// @name Common-Codes
/// @brief Common LSB exit codes for all actions except "status".
//@{
const LSBExitCode LSB_RC_SUCCESS          = 0; //!< success
const LSBExitCode LSB_RC_FAILURE          = 1; //!< generic or unspecified error
const LSBExitCode LSB_RC_INVALID_ARGUMENT = 2; //!< invalid or excess argument(s)
const LSBExitCode LSB_RC_NOT_IMPLEMENTED  = 3; //!< unimplemented feature ("reload")
const LSBExitCode LSB_RC_NOT_ALLOWED      = 4; //!< user had insufficient privilege
const LSBExitCode LSB_RC_NOT_INSTALLED    = 5; //!< program is not installed
const LSBExitCode LSB_RC_NOT_CONFIGURED   = 6; //!< program is not configured
const LSBExitCode LSB_RC_NOT_RUNNING      = 7; //!< program is not running
//@}

/// @name Status-Codes
/// @brief Common LSB exit codes for the "status" action.
//@{
const LSBExitCode LSB_ST_RUNNING          = 0; //!< program is running or service is OK
const LSBExitCode LSB_ST_DEAD_PID_EXISTS  = 1; //!< program is dead and /var/run pid file exists
const LSBExitCode LSB_ST_DEAD_LOCK_EXISTS = 2; //!< program is dead and /var/lock lock file exists
const LSBExitCode LSB_ST_NOT_RUNNING      = 3; //!< program is not running
const LSBExitCode LSB_ST_UNKNOWN          = 4; //!< program or service status is unknown
//@}


typedef blocxx::Map<blocxx::String, blocxx::String> LSBInitInfo;


// -------------------------------------------------------------------
/**
 * @brief LSB specific sevice class.
 *
 * This class provides a interface to LSB compliant runlevel scripts.
 */
class LSBService: public Service
{
public:
	/**
	 * Construct a LSBService object using the specified name.
	 *
	 * @param name The name of the service or a full path to the script.
	 * @throws limal::ValueError if name is empty or invalid.
	 */
	LSBService(const blocxx::String &name);


	/**
	 * Destructor of LSBService.
	 */
	~LSBService();


	/**
	 * @return True, if the service is enabled in any runlevel.
	 */
	bool
	isServiceEnabled() const;


	/**
	 * @param  runlevel The runlevel to check for.
	 * @return True, if the service script is enabled
	 *         (link exists) in the specified runlevel.
	 */
	bool
	isServiceEnabledInRunlevel(Runlevel runlevel) const;


	/**
	 * Enable the service using its default runlevel set.
	 *
	 * @return true, if the service was sucesfully enabled.
	 */
	bool enableService() const;


	/**
	 * Disable the service.
	 * @return true, if the service was sucesfully disabled.
	 */
	bool disableService() const;


	/**
	 * Returns a LSBInitInfo map containing the service information
	 * tags. Leading and trailing spaces in the strings are stripped.
	 * The Description field may contain multiple lines, that are
	 * separated by \n in the map's value string.
	 *
	 * @return The INIT INFO tags from service script.
	 */
	LSBInitInfo
	getServiceInitInfo() const;


	/**
	 * Returns known LSB action names.
	 *
	 * By default, alias to the getDefaultActionNames() method;
	 * may be overiden by a specialized class.
	 *
	 * @return Array of known action names.
	 */
	blocxx::Array<blocxx::String>
	getServiceActions() const;


	/**
	 * Returns array of default LSB action names.
	 *
	 * LSB defines in Section 8.2, that the init script actions
	 * "start", "stop", "restart", "force-reload" and "status"
	 * shall be supported by all init scripts, "try-restart",
	 * "reload" and other are optional.
	 *
	 * @return The default list of action names for the
	 *         current system.
	 */
	blocxx::Array<blocxx::String>
	getDefaultActions() const;


	/**
	 * Execute the service script action.
	 *
	 * This method uses the specified additional arguments
	 * (the 0th argument is always passed) and execution
	 * environment.
	 *
	 * The stdout and stderr outputs are gathered into the
	 * specified stdOutput and errOutput variables.
	 * If the envVars map is empty, the environment of the
	 * current process is used.
	 *
	 * It returns the LSB exit code returned by the script,
	 * if the script terminated normally or -1 in any other
	 * case (killed, core dumped).
	 *
	 * If a timeout or output limit is reached, the process
	 * will be killed and an exception is thrown.
	 *
	 * See blocxx::Exec::executeProcessAndGatherOutput()
	 * methods documentation for more informations.
	 *
	 * @param  action      The action name (first argument).
	 * @param  cmdArgs     Additional script arguments.
	 * @param  stdOutput   String reference for the standard output.
	 * @param  errOutput   String reference for the error output.
	 * @param  envVars     The environment to use (empty for current).
	 * @param  stdInput    String with the script input.
	 * @param  timeoutSecs The number of seconds to wait for
	 *         the process to exit or -1 to wait infinitely.
	 * @param  outputLimit The number of bytes that stdOutput
	 *         or errOutput should not exceed or -1 for unlimited.
	 *         the process to exit or -1 to wait infinitely.
	 *
	 * @return The LSB exit status code or -1 in case that
	 *         the process was killed or core dumped.
	 *
	 * @throws limal::ValueException on invalid / empty action name.
	 * @throws blocxx::ExecErrorException on execution errors.
	 * @throws blocxx::ExecTimeoutException if the script hasn't
	 *        finished within timeoutSecs.
	 * @throws blocxx::ExecBufferFullException if the output of
	 *        the script exceeds the specified outputLimit.
	 */
	LSBExitCode
	executeAction(const blocxx::String   &action,
	              const ServiceCmdArgs   &cmdArgs,
	              blocxx::String         &stdOutput,
	              blocxx::String         &errOutput,
	              const ServiceEnvVars   &envVars,
	              const blocxx::String   &stdInput,
	              int                    timeoutSecs,
	              int                    outputLimit);


	/**
	 * Execute the service script action.
	 *
	 * This method discards the output of the script,
	 * uses the environment of the current process and
	 * supports additional arguments.
	 *
	 * @note This function may run infinitely if the
	 * script never ends.
	 *
	 * @param  action  The action name.
	 * @param  cmdArgs Additional script arguments.
	 * @return The LSB exit status or -1.
	 * @throws limal::ValueException on invalid / empty action name.
	 * @throws blocxx::ExecErrorException on execution errors.
	 */
	LSBExitCode
	executeAction(const blocxx::String   &action,
	              const ServiceCmdArgs   &cmdArgs);


	/**
	 * Execute the service script action.
	 *
	 * This method discards the output of the script and
	 * uses the environment of the current process.
	 *
	 * @note This function may run infinitely if the
	 * script never ends.
	 *
	 * @param  action  The action name.
	 * @return The LSB exit status or -1.
	 * @throws limal::ValueException on invalid / empty action name.
	 * @throws blocxx::ExecErrorException on execution errors.
	 */
	LSBExitCode
	executeAction(const blocxx::String    &action);


	/**
	 * @brief Deprecated execute the service script action variant.
	 *
	 * @deprecated This variant is deprecated, because it
	 * exposes the non-portable raw exitStatus parameter,
	 * that is not avaliable in blocxx 2.x version, where
	 * it is always set to -1.
	 */
	LSBExitCode
	executeAction(const blocxx::String   &action,
	              const ServiceCmdArgs   &cmdArgs,
	              blocxx::String         &stdOutput,
	              blocxx::String         &errOutput,
	              int                    &execStatus,
	              const ServiceEnvVars   &envVars,
	              const blocxx::String   &stdInput,
	              int                    timeoutSecs,
	              int                    outputLimit)
	              // BLOCXX_DEPRECATED
	              ;


protected:
	void
	addServiceAction(const blocxx::String &action);

private:
	blocxx::Array<blocxx::String> m_actions;
};


// -------------------------------------------------------------------
}      // End of RUNLEVEL_NAMESPACE
}      // End of LIMAL_NAMESPACE

#endif // LIMAL_RUNLEVEL_LSB_SERVICE_HPP
/* vim: set ts=8 sts=8 sw=8 ai noet: */
