/*---------------------------------------------------------------------\
|                                                                      |
|                     _     _   _   _     __     _                     |
|                    | |   | | | \_/ |   /  \   | |                    |
|                    | |   | | | |_| |  / /\ \  | |                    |
|                    | |__ | | | | | | / ____ \ | |__                  |
|                    |____||_| |_| |_|/ /    \ \|____|                 |
|                                                                      |
|                             runlevel library                         |
|                                                                      |
|                                         (C) SUSE Linux Products GmbH |
\----------------------------------------------------------------------/

  File:       Runlevel.hpp

  Author:     Marius Tomaschewski
  Maintainer: Marius Tomaschewski

  Purpose:

/-*/
/**
 * @file   Runlevel.hpp
 * @brief  Runlevel utility functions.
 */
#ifndef   LIMAL_RUNLEVEL_HPP
#define   LIMAL_RUNLEVEL_HPP

#include <limal/runlevel/config.h>
#include <blocxx/String.hpp>
#include <blocxx/Array.hpp>


// -------------------------------------------------------------------
namespace LIMAL_NAMESPACE
{
namespace RUNLEVEL_NAMESPACE
{


/**
 * Runlevel numbers are ascii charactes @c '0' - @c '6', @c 'B' and @c 'S'.
 *
 * Linux Standard Base Core Specification 3.0.1 defines the set of valid
 * runlevels @c 0 - @c 6, reflecting the most frequent existing practice:
 *
 * - @c 0	System halt
 * - @c 1	Single user mode
 * - @c 2	Local multiuser without remote network
 * - @c 3	Full multiuser with network
 * - @c 4	Not used (reserved for local use)
 * - @c 5	Full multiuser with network and X11 or equivalent
 * - @c 6	System reboot (Do not use this as default runlevel!)
 *
 * @note The meaning of further runlevels is implementation specific.
 * 
 * See the inittab(5) and init(8) manual pages for more information.
 *
 * The runlevel @c S is not meant to be used directly. It indicates,
 * that the system is entering single user mode.
 *
 * The runlevel @c B is a ondemand entry in the /etc/inittab and can be
 * used to execute processes on demand without to change the runlevel.
 *
 * On NOVELL / SUSE LINUX, services marked with runlevel @c B in LSB
 * INIT INFO tags will be executed at system boot.
 *
 * The runlevels @c 7 - @c 9 and the ondemand entries @c A and @c C may
 * be also valid, but usually unused.
 */
typedef char                        Runlevel;
typedef blocxx::Array<Runlevel>     Runlevels;


/**
 * @return Default runlevel as configured in /etc/inittab
 *         file or -1 on error.
 */
Runlevel
getDefaultRunlevel();


/**
 * @return Currently active runlevel or -1 on error.
 */
Runlevel
getCurrentRunlevel();


/**
 * @return List of runlevels enabled in /etc/inittab.
 */
Runlevels
getEnabledRunlevels();


/**
 * @param runlevel The runlevel to check.
 * @return True, if the runlevel is enabled.
 */
bool
isRunlevelEnabled(Runlevel runlevel);


/**
 * Set the default runlevel in /etc/inittab.
 *
 * @param runlevel The new runlevel to set.
 * @return True on successfull change, false on failure.
 */
bool
setDefaultRunlevel(Runlevel runlevel);


/**
 * Trigger a switch to the specified runlevel.
 * @warning e.g. runlevel 0 will halt the system!
 *
 * @param runlevel The new runlevel to switch to.
 * @return True on successfull request execution, false on failure.
 */
bool
switchToRunlevel(Runlevel runlevel);


// -------------------------------------------------------------------
/**
 * @return The list of service script names avaliable
 *         (active) for the specified runlevel.
 */
blocxx::Array<blocxx::String>
getRunlevelServiceNames(Runlevel runlevel);


// -------------------------------------------------------------------
}      // End of RUNLEVEL_NAMESPACE
}      // End of LIMAL_NAMESPACE
#endif // LIMAL_RUNLEVEL_HPP
