/*---------------------------------------------------------------------\
|                                                                      |
|                     _     _   _   _     __     _                     |
|                    | |   | | | \_/ |   /  \   | |                    |
|                    | |   | | | |_| |  / /\ \  | |                    |
|                    | |__ | | | | | | / ____ \ | |__                  |
|                    |____||_| |_| |_|/ /    \ \|____|                 |
|                                                                      |
|                             runlevel library                         |
|                                                                      |
|                                         (C) SUSE Linux Products GmbH |
\----------------------------------------------------------------------/

  File:       Service.hpp

  Author:     Marius Tomaschewski
  Maintainer: Marius Tomaschewski

  Purpose:

/-*/
/**
 * @file   Service.hpp
 * @brief  This is a short description of the library.
 */
#ifndef   LIMAL_RUNLEVEL_SERVICE_HPP
#define   LIMAL_RUNLEVEL_SERVICE_HPP

#include <limal/runlevel/config.h>
#include <blocxx/String.hpp>
#include <blocxx/Array.hpp>
#include <blocxx/Map.hpp>


// -------------------------------------------------------------------
namespace LIMAL_NAMESPACE
{
namespace RUNLEVEL_NAMESPACE
{

// -------------------------------------------------------------------
/**
 * @return The default service script path of the
 *         current system, e.g. "/etc/init.d".
 */
blocxx::String
getServiceScriptDir();


// -------------------------------------------------------------------
/**
 * @return The list of service script names installed
 *         in the script path of the current system.
 */
blocxx::Array<blocxx::String>
getInstalledServiceNames();


// -------------------------------------------------------------------
/**
 * @brief Shortcut for command argument array.
 */
typedef blocxx::Array<blocxx::String>               ServiceCmdArgs;


/**
 * @brief Shortcut for environment variable map.
 */
typedef blocxx::Map<blocxx::String,blocxx::String>  ServiceEnvVars;


// -------------------------------------------------------------------
/**
 * @brief Generic service class.
 *
 * This class provides a generic interface to service scripts.
 */
class Service
{
public:
	/**
	 * Construct a Service object using the specified name.
	 *
	 * @param name The name of the service or a full path to the script.
	 * @throws limal::ValueError if name is empty or invalid.
	 */
	Service(const blocxx::String &name);

	/**
	 * Destructor of Service.
	 */
	~Service();

	/**
	 * @return The short name of the service.
	 */
	blocxx::String
	getServiceName() const;

	/**
	 * @return The service script directory name.
	 */
	blocxx::String
	getScriptPath() const;

	/**
	 * @return Full path to the script file name.
	 */
	blocxx::String
	getScriptName() const;


	/**
	 * @return True, if the service script exists
	 *         in the system service directory.
	 */
	bool
	isServiceInstalled() const;


	/**
	 * Execute the service script.
	 *
	 * This method uses the specified additional arguments
	 * (the 0th argument is always passed) and execution
	 * environment.
	 *
	 * The stdout and stderr outputs are gathered into the
	 * specified stdOutput and errOutput variables.
	 * If the envVars map is empty, the environment of the
	 * current process is used.
	 *
	 * It returns the exit status of the script process,
	 * if the script terminated normally or -1 in any other
	 * case (killed, core dumped).
	 *
	 * If a timeout or output limit is reached, the process
	 * will be killed and an exception is thrown.
	 *
	 * See blocxx::Exec::executeProcessAndGatherOutput()
	 * methods documentation for more informations.
	 *
	 * @param  cmdArgs     Additional script arguments.
	 * @param  stdOutput   String reference for the standard output.
	 * @param  errOutput   String reference for the error output.
	 * @param  envVars     The environment to use (empty for current).
	 * @param  stdInput    String with the script input.
	 * @param  timeoutSecs The number of seconds to wait for
	 *         the process to exit or -1 to wait infinitely.
	 * @param  outputLimit The number of bytes that stdOutput
	 *         or errOutput should not exceed or -1 for unlimited.
	 *         the process to exit or -1 to wait infinitely.
	 *
	 * @return The exit status of the process or -1 in case
	 *         that the process was killed or core dumped.
	 *
	 * @throws blocxx::ExecErrorException on execution errors.
	 * @throws blocxx::ExecTimeoutException if the script hasn't
	 *         finished within timeoutSecs.
	 * @throws blocxx::ExecBufferFullException if the output of
	 *         the script exceeds the specified outputLimit.
	 */
	int
	executeScript(const ServiceCmdArgs   &cmdArgs,
	              blocxx::String         &stdOutput,
	              blocxx::String         &errOutput,
	              const ServiceEnvVars   &envVars,
	              const blocxx::String   &stdInput,
	              int                    timeoutSecs,
	              int                    outputLimit);


	/**
	 * Execute the service script.
	 *
	 * This method discards the output of the script,
	 * uses the environment of the current process and
	 * supports additional arguments.
	 *
	 * @note This function may run infinitely if the
	 * script never ends.
	 *
	 * @param  cmdArgs Additional script arguments.
	 * @return The process exit status or -1 on failure.
	 * @throws blocxx::ExecErrorException on execution errors.
	 */
	int
	executeScript(const ServiceCmdArgs &cmdArgs);


	/**
	 * @brief Deprecated execute the service script variant.
	 *
	 * @deprecated This variant is deprecated, because it
	 * exposes the non-portable raw exitStatus parameter,
	 * that is not avaliable in blocxx 2.x version, where
	 * it is always set to -1.
	 */
	int
	executeScript(const ServiceCmdArgs   &cmdArgs,
	              blocxx::String         &stdOutput,
	              blocxx::String         &errOutput,
	              int                    &execStatus,
	              const ServiceEnvVars   &envVars,
	              const blocxx::String   &stdInput,
	              int                    timeoutSecs,
	              int                    outputLimit)
	              // BLOCXX_DEPRECATED
	              ;


protected:
	/**
	 * Set the service name and optionally the script path.
	 *
	 * @param name The name of the service or a full path to the script.
	 * @throws limal::ValueError if name is empty or invalid.
	 */
	void
	setScriptName(const blocxx::String &name);

private:
	// disallow, we allways need a name.
	Service();

	blocxx::String  m_path;
	blocxx::String  m_name;
};


// -------------------------------------------------------------------
}      // End of RUNLEVEL_NAMESPACE
}      // End of LIMAL_NAMESPACE

#endif // LIMAL_RUNLEVEL_SERVICE_HPP
